/*****************************************************
Date    : 23.08.2011
Author  : Evildesignmail.ru

Chip type               : ATtiny13A
AVR Core Clock frequency: 4,800000 MHz
*****************************************************/

#include <tiny13a.h>


//     .   < 650 .  
#define TIME_LIGHT_ON 25U  

//   
#define LAMP PORTB.0    

//voltage limit in relative units for six measurements
#define TH_V 60 


volatile unsigned char phase_tick;               //    100   -   
volatile unsigned char max_measured_phase_tick;  //       100     
volatile unsigned char measured_phase_tick;      //        100     
volatile unsigned char light_on;                 // -           0  max_measured_phase_tick
volatile unsigned char phase_locked;             //      
volatile unsigned int num_half_sin;              //           

// Pin change interrupt service routine
//    
interrupt [PC_INT0] void pin_change_isr(void)
{
 //     -   
 if(phase_locked==0) return;
 if(PINB.2==1) return; //   .     .
 if(light_on==0) light_on=1;
 num_half_sin=0;
}


// Timer 0 output compare A interrupt service routine
// Every 100 us start ADC
interrupt [TIM0_COMPA] void timer0_compa_isr(void)
{
 //   
 phase_tick++;
 if(phase_tick>=max_measured_phase_tick)
   {
    phase_tick=0;
    num_half_sin++;
   }
   
 if(measured_phase_tick<115)measured_phase_tick++;  
 
   
 
 if(light_on) 
   { 
    //          
    if(phase_tick==0)
      { 
       //      2500   2500*10ms=25 ,  
       if(num_half_sin>=(TIME_LIGHT_ON*100U))
         {  
          //         .   .
          if(num_half_sin==(TIME_LIGHT_ON*100U)) GIMSK=(0<<PCIE);
          light_on--;  
          //        
          if(light_on==0)
            {
             GIFR=(1<<PCIF);
             GIMSK=(1<<PCIE);
            }
         } else
                //      
                if(light_on<max_measured_phase_tick-10)light_on++;

      }   
    //        
    if (phase_tick==max_measured_phase_tick-light_on)
       {
        if(light_on<=10) LAMP=0;
           else LAMP=1;
       }  
    if (phase_tick>=max_measured_phase_tick-10) LAMP=0;
   } else 
          LAMP=0;
         
}

// Analog Comparator interrupt service routine
//       MOSFET
//     
interrupt [ANA_COMP] void ana_comp_isr(void)
{
 if(light_on) 
   {
    light_on=0;
    num_half_sin=0;
    phase_locked=0;
    LAMP=0;
   }
}


// ADC interrupt service routine
interrupt [ADC_INT] void adc_isr(void)
{
 static unsigned char adc_prev;
 static unsigned char flag_direction_up; //    
 static unsigned char num_up_samples;    //    
 static unsigned char num_down_samples;  //    
 
 unsigned char adc_data;
 // Read the 8 most significant bits
 // of the AD conversion result
 adc_data=ADCH;
 
 if((adc_data<TH_V)&&(LAMP==0))
   { 
    if (flag_direction_up==0)
       {
        if (adc_data<=adc_prev)
           {
            num_down_samples++;
           } else 
                 {                
                  if(num_down_samples>=5)
                    {
                     num_up_samples=1;
                     flag_direction_up=1;
                    } else
                          {
                           num_up_samples=0;
                           num_down_samples=0;
                           flag_direction_up=0;
                          }
                     
                 } 
        } else
              {
               if (adc_data>=adc_prev)
                  {
                   num_up_samples++;         
                   if(num_up_samples==5)
                     {
                      //if the frequency 47,5 to 52,5 Hz, phase locking
                      if((measured_phase_tick>=95)&&(measured_phase_tick<=105))
                        {
                         phase_tick=5;
                         // average                                                                                                       
                         max_measured_phase_tick=(max_measured_phase_tick+(measured_phase_tick-5))>>1;
                         phase_locked=1;
                        }
                      measured_phase_tick=5;
                      num_up_samples=0;
                      num_down_samples=0;
                      flag_direction_up=0; 
                     }
                  } else 
                        { 
                         num_up_samples=0;
                         num_down_samples=0;
                         flag_direction_up=0;
                        }
              }    
   } else 
          {
                  num_up_samples=0;
                  num_down_samples=0;
                  flag_direction_up=0;
          }
 adc_prev=adc_data; 
}



void main(void)
{
// Input/Output Ports initialization
// Port B initialization
// Func5=In Func4=In Func3=In Func2=In Func1=In Func0=Out 
// State5=T State4=T State3=T State2=T State1=T State0=0 
PORTB=0x00;
DDRB=(1<<DDB0);

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 600,000 kHz
// Mode: CTC top=OCR0A
// OC0A output: Disconnected
// OC0B output: Disconnected
TCCR0A=(1<<WGM01);
TCCR0B=(1<<CS01);
TCNT0=0x00;
OCR0A=0x3C;
OCR0B=0x00;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=(1<<OCIE0A);

// Analog Comparator initialization
// Analog Comparator: On
// The Analog Comparator's positive input is
// connected to the Bandgap Voltage Reference
// Interrupt on Falling Output Edge
ACSR=(1<<ACBG)|(1<<ACIE)|(1<<ACIS1);
ADCSRB=0x00;

// ADC initialization
// ADC Clock frequency: 300,000 kHz
// ADC Bandgap Voltage Reference: On
// ADC Auto Trigger Source: TIM0_COMPA
// Only the 8 most significant bits of
// the AD conversion result are used
// Digital input buffers on ADC0: Off, ADC1: Off, ADC2: On, ADC3: Off
DIDR0  = (1<<ADC0D) |  (1<<ADC3D) | (1<<ADC1D) | (1<<AIN1D) | (1<<AIN0D);

ADMUX  = (1<<REFS0) | (1<<ADLAR) | (1<<MUX1)  | (1<<MUX0) ;
ADCSRA = (1<<ADEN)  | (1<<ADATE) | (1<<ADIE)  | (1<<ADPS2);
ADCSRB = (1<<ADTS1) | (1<<ADTS0);

// Interrupt on any change on pin PCINT4
PCMSK=(1<<PCINT4);
GIMSK=(1<<PCIE);

ACSR=(1<<ACI);
TIFR0=(1<<OCF0B)|(1<<OCF0A)|(1<<TOV0);
GIFR=(1<<PCIF);
MCUCR=0x00;

max_measured_phase_tick=100; // 100samlpes*100us=10ms     100   

// Global enable interrupts
#asm("sei")

for(;;);
}